@@ -1,10 +1,10 @@
 # Copyrights 2013 by [Mark Overmeer].
 #  For other contributors see Changes.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.00.
+# Pod stripped from pm file by OODoc 2.01.
 package Net::OAuth2::AccessToken;
 use vars '$VERSION';
-$VERSION = '0.51';
+$VERSION = '0.52';
 
 use warnings;
 use strict;
@@ -20,7 +20,7 @@ my @session = qw/access_token token_type refresh_token expires_at
    scope state auto_refresh/;
 
 # This class name is kept for backwards compatibility: a better name
-# would have been: Net::OAuth2::BearerToken
+# would have been: Net::OAuth2::Session, with a ::Token::Bearer split-off.
 
 # In the future, most of this functionality will probably need to be
 # split-off in a base class ::Token, to be shared with a new extension
@@ -45,6 +45,7 @@ sub init($)
     $self->{NOA_scope}         = $args->{scope};
     $self->{NOA_token_type}    = $args->{token_type};
     $self->{NOA_auto_refresh}  = $args->{auto_refresh};
+    $self->{NOA_changed}       = $args->{changed};
 
     $self->{NOA_error}         = $args->{error};
     $self->{NOA_error_uri}     = $args->{error_uri};
@@ -61,18 +62,27 @@ sub session_thaw($%)
 
 #--------------
 
-
 sub token_type() {shift->{NOA_token_type}}
 sub scope()      {shift->{NOA_scope}}
 sub profile()    {shift->{NOA_profile}}
 
 
+sub changed(;$)
+{   my $s = shift; @_ ? $s->{NOA_changed} = shift : $s->{NOA_changed} }
+
+
 sub access_token()
 {   my $self = shift;
 
-    $self->refresh
-        if  $self->auto_refresh
-        || ($self->refresh_token && $self->expired);
+    if($self->expired)
+    {   delete $self->{NOA_access_token};
+        $self->{NOA_changed} = 1;
+        $self->refresh if $self->auto_refresh;
+    }
+    elsif($self->refresh_token)
+    {   # refresh token at each use
+        $self->refresh;
+    }
 
     $self->{NOA_access_token};
 }
@@ -105,9 +115,9 @@ sub expired(;$)
 
 sub update_token($$$)
 {   my ($self, $token, $type, $exp) = @_;
-    $self->{NOA_access_token}   = $token;
-    $self->{NOA_token_type}    = $type if $type;
-    $self->{NOA_expires_at} = $exp;
+    $self->{NOA_access_token} = $token;
+    $self->{NOA_token_type}   = $type if $type;
+    $self->{NOA_expires_at}   = $exp;
     $token;
 }
 
@@ -120,10 +130,11 @@ sub to_json()
 *to_string = \&to_json;  # until v0.50
 
 
-sub freeze(%)
+sub session_freeze(%)
 {   my ($self, %args) = @_;
     my %data    = (net_oauth2_version => $VERSION);
     defined $self->{"NOA_$_"} && ($data{$_} = $self->{"NOA_$_"}) for @session;
+    $self->changed(0);
     \%data;
 }
 
